/* FILE: DynWaitlist.cpp
 *
 * WHAT: Implementation of the DynWaitlist class.
 *       See DynWaitlist.h for class description
 */

#include <DynWaitlist.h>


DynWaitlistImpl::DynWaitlistImpl(  
    WORD nMaxHandles,  // number of handles
    HANDLE *pHandles,   // pointer to array of handle slots
    WORD *pIds,         // pointer to array of IDs
    WORD wFirstID)      // value of first ID to use
// Class Constructor.  Initializes object state
:  m_nMaxHandles(nMaxHandles)
,  m_pHandles(pHandles)
,  m_pIds(pIds)
,  m_nHandles(0)
{
    // fill the pool of available IDs
    WORD wId = wFirstID;
    for( WORD i = 0; i < nMaxHandles; ++i )
    {
        m_pIds[i] = wId;
        wId++;
    }
}


BOOL DynWaitlistImpl::Add(
    HANDLE hNewHandle, // handle to be added
    WORD *pwNewId ) // OUT parameter - value of new ID picked
// adds one element to the array of handles
{
    if( m_nHandles >= m_nMaxHandles )
    {
        // no more room, no can do
        return FALSE;
    }
    m_pHandles[ m_nHandles ] = hNewHandle;

    // pick the first available ID
    (*pwNewId) = m_pIds[ m_nHandles ];

    ++m_nHandles;
    return TRUE;
}

BOOL DynWaitlistImpl::Remove(
    WORD wId ) // ID of handle being removed
// removes one element from the array of handles
{
    WORD i;
    BOOL bFoundIt = FALSE;
    for( i = 0; i < m_nHandles; ++i )
    {
        // if we found the one we want to remove
        if( m_pIds[i] == wId )
        {
            // found it!
            bFoundIt = TRUE;
            break;
        }
    }

    // found the ID we were looking for?
    if( bFoundIt )
    {
        WORD wMaxIdx = (m_nHandles - 1);
        if( i < wMaxIdx ) // if it isn't the last item being removed
        {
           // take what used to be the last item and move it down,
           // so it takes the place of the item that was deleted
           m_pIds    [i] = m_pIds    [ wMaxIdx ];
           m_pHandles[i] = m_pHandles[ wMaxIdx ];

           // save the ID being removed, so it can be reused in a future Add
           m_pIds    [ wMaxIdx ] = wId;
        }
        --m_nHandles;
        m_pHandles[m_nHandles] = 0;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}


DWORD DynWaitlistImpl::TranslateWaitResult(
    DWORD dwWaitResult ) // value returned by WaitForMultipleObjectsXXX
// translates the index-based value returned by Windows into an ID-based value for comparison
{

    if( (dwWaitResult >= WAIT_OBJECT_0) && 
        (dwWaitResult < (DWORD)(WAIT_OBJECT_0 + m_nHandles) ) )
    {
        return HANDLE_SIGNALED | m_pIds[dwWaitResult - WAIT_OBJECT_0];
    }
    if( (dwWaitResult >= WAIT_ABANDONED_0) && 
        (dwWaitResult < (DWORD)(WAIT_ABANDONED_0 + m_nHandles) ) )
    {
        return HANDLE_ABANDONED | m_pIds[dwWaitResult - WAIT_ABANDONED_0];
    }

    return dwWaitResult; // no translation needed for other values
}
